/******************************
 * RomArea.c
 *
 * Steve Garcia -- 8 Sep 1995 -- Vers 0.1
 *
 * Reads a listing of directories (with descriptions) and creates
 * a ROMAREAS.DAT that is compatible with RomDoor 4.12
 *
 * 14 Nov 1995 -- Vers 0.5
 * Still crude, but does not have starting and stopping columns hardwired
 * for specific numbers.  Might not have to recompile for each CD!
 *
 * 16 Nov 1995 -- Vers 0.6
 * Fixed bugs introduced in generalization.  Knowing exactly where each
 * string starts and stops is _such_ a simplification!
 *
 * 17 Nov 1995 -- Vers 0.7
 * Discovered tech note 7.  Gotta upcase the path!
 *
 *****************************/

 /*
 Note: The RomDoor Area file is limited to 30 characters.  Edit your
       input file!!
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <alloc.h>

#define LINELENGTH  512
#define VERSNO "0.7"

/*****************************
 *  RomArea.DAT has the following format:
 *  1) The file reserves 443 bytes for each file area.  It is filled with
 *     nulls until an area is defined, and all reserved but unused areas
 *     are filled with nulls.
 *  2) Each string is preceded by its length, expressed as a single byte.
 *     Presumably RomDoor was written in Pascal.
 *  3) Each record has an identical structure, 443 bytes long.
 *  4) The first field in the record is a 30 character string, null padded
 *     on the right (total 31 bytes with length byte).
 *  5) The next field in the record is a 67 character string which
 *     represents the first path for the area. (68 bytes with length).
 *  6) The remaining 5 path fields appear to _average_ 68 bytes long.
 *     I may have miscounted, but it looked to me like they went:
 *     69, 68, 67, 68, 73
 *     Don't ask me what is going on with that last 5 bytes in the last
 *     field.  It may be used for something else and not related to the
 *     last path field.  Groups maybe?
 *  7) The paths MUST be in uppercase and have a trailing backslash.  All
 *     the RomDoor utilities will work if this is not the case, but the
 *     program will not find the files on the disc.
 *
 *  We will completely ignore the last five path fields, since we will
 *  assume that we want to create an area with description for each
 *  directory in the list.
 *
 *****************************/

struct RomRec {
      char namelen;
      char name[30];
      char pathlen;
      char path[67];
      char blnk[344];
   };
typedef struct RomRec ROMREC;

int  main(int argc, char *argv[]);
void prop(void);
int  get_area_name(char *instring, char *areaname);
int  get_path_name(char *instring, char *pathname);
char *clean_nl(char *string);
char *strltrim(char *str);
int main(int argc, char *argv[])
{
   FILE *filin = NULL, *filout = NULL;
   char temp[LINELENGTH],
        instring[LINELENGTH],
        *p_pnt;
   int  i = 0;      /* number of areas read in */
   ROMREC *area;

   printf("RomDoor Area Creator %s,  %s\n", VERSNO, __DATE__);
   printf("Creates ROMAREAS.DAT from Directory Description Listing\n");
   printf("Usage: RomArea <filename>\n");


   if ((filin = fopen( argv[1], "rt")) == NULL)
   {
      printf("Unable to open %s for input.\n", argv[1]);
      exit(2);
   }
   else
      printf("Opening %s for input.\n", argv[1]);


      /* Important -- open output in binary mode */
   if ((filout = fopen("romareas.dat", "wb")) == NULL)
   {
      printf("Unable to open RomAreas.Dat for output.\n");
      exit(3);
   }
   else
      printf("Creating RomAreas.Dat\n ");


   while ((fgets(instring, LINELENGTH, filin)) != NULL)
   {
      /* Create a new ROMAREA record -- we use calloc to zero the memory */
      area = (ROMREC *) calloc(443,1);



      if (!get_path_name(instring, area->path))
         printf("Error getting path!\n");
      else
         area->pathlen = (char) min(strlen(area->path), 67);



      if (!get_area_name(instring, area->name))
         printf("Error getting area name!\n");
      else
      {
         area->namelen = (char)min(strlen(area->name), 30);
      }



      prop();
      i++;

      /* write out data and clear the record */
      fwrite(area, 1, 443, filout);
      free (area);
   }
   fclose(filin);
   fclose(filout);

   printf("\n%d areas were created!\n", i);
   return 0;
}

void prop()
{
   static posit = 0;

   switch (posit)
   {
   case 0: printf("\b|"); posit++; break;
   case 1: printf("\b/"); posit++; break;
   case 2: printf("\b-"); posit++; break;
   case 3: printf("\b\\"); posit = 0; break;
   }
}

int get_area_name(char *instring, char *name)
{
   int result = 0;
   char *p_str = NULL,
        dumm[31];

   strltrim(instring);
   p_str = (char *)strstr(instring, " ");
   strltrim(p_str);
   result = strncpy(dumm, p_str, 30);
   dumm[30] = '\0';
   clean_nl(dumm);
   strncpy(name, dumm, 30);

   return result;
}



int get_path_name(char *instring, char *path)
{
   int result = 0,
       i,
       length = 0;
   char temp[50];


   if (*instring != '\\')
      *path = '\\';

   strltrim(instring);
   /* force to uppercase */
   sscanf(instring, "%s", temp);
   length = strlen(temp);
   for (i=0; i < length; i++)
   {
      temp[i] = toupper(temp[i]);
   }
   if (strcmp(temp[length-1], '\\'))
      strcat(temp, "\\");

   result = strncat(path, temp, 48);

   return result;
}

/********************************/
/* clean_nl() */
/* cleans the newline character off the end of the string, if it exists
*/
char *clean_nl(char *string)
{
   int len;

   if (string[len = strlen(string) - 1] == '\n')
      string[len] = '\0';
   return string;
}




/******************************************
 * returns address of string with         *
 * leading spaces removed                 *
 ******************************************/
char   *strltrim(char *str)
{
   char *whitespace = " \t\n\r",
        *temp;

   temp = str + strspn(str, whitespace);
   strcpy(str, temp);

   return str;
}

